package com.displaynone.acss.components.auth.internal_utils;

import android.content.Context;
import android.content.SharedPreferences;

import androidx.annotation.Nullable;
import androidx.security.crypto.EncryptedSharedPreferences;
import androidx.security.crypto.MasterKeys;

import com.displaynone.acss.components.auth.models.user.repository.dto.UserDTO;
import com.google.gson.Gson;

import java.io.IOException;
import java.security.GeneralSecurityException;

public class UserManager {
    // Preferences
    private static final String _PREFERENCES_FILENAME = "userData";
    private final SharedPreferences _preferences;

    // Utils
    private static final Gson gson = new Gson();

    // Keys
    private static final String _KEY_USER = "user";

    // Cache
    private UserDTO userDTO;

    public UserManager(Context context) {
        this._preferences = this._createEncryptedPreferences(context);
    }

    private SharedPreferences _createEncryptedPreferences(Context ctx) {
        try {
            return EncryptedSharedPreferences.create(
                    _PREFERENCES_FILENAME,
                    MasterKeys.getOrCreate(MasterKeys.AES256_GCM_SPEC),
                    ctx,
                    EncryptedSharedPreferences.PrefKeyEncryptionScheme.AES256_SIV,
                    EncryptedSharedPreferences.PrefValueEncryptionScheme.AES256_GCM
            );
        } catch (GeneralSecurityException | IOException e) {
            throw new RuntimeException(e);
        }
    }

    public void saveDto(UserDTO userDTO) {
        this.userDTO = userDTO;
        _preferences.edit()
                .putString(_KEY_USER, toJson(userDTO))
                .apply();
    }

    public @Nullable UserDTO getDto() {
        if (this.userDTO != null) return this.userDTO;
        return fromJson(_preferences.getString(_KEY_USER, null));
    }

    private UserDTO fromJson(String userJSON) {
        return gson.fromJson(userJSON, UserDTO.class);
    }

    private String toJson(UserDTO userDTO) {
        return gson.toJson(userDTO);
    }
}
